/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */

package com.ibm.jps.uxtoweb.utility;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.eclipse.emf.codegen.jmerge.JControlModel;
import org.eclipse.emf.codegen.jmerge.JMerger;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.osgi.framework.Bundle;

/**
 * An utility class to contain methods to manipulate Eclipse resources.
 */
public class ResourceUtil {
	public static IProject jetProject = null;

	/**
	 * Constructor. Being a utility class with all static methods and no state
	 * of its own, this class does not need its constaructor to be public.
	 */
	public ResourceUtil() {
		super();
	}

	/**
	 * Returns the handle to a file in a folder inside a project. The file or
	 * the folder may not yet exist.
	 * 
	 * Given a project this method creates and returns a handle to a identified
	 * file. The folder or the file may not yet exist. If the identified folder
	 * does not exist, the method creates the folder under the project, and then
	 * creates a file handle to the file.
	 * 
	 * @param project
	 *            Handle to the project containing the folder,
	 *            <code>folderName</code> and the file <code>fileName</code>
	 * @param folderName
	 *            The name of the folder containing the file
	 *            <code>filename</code>
	 * @param fileName
	 *            The name of the file
	 * @return A handle to the file. If the file does not exist in the folder
	 *         returnvalue.exists() will false.
	 * @throws CoreException
	 *             If the creation of the folder (in the case it does not exist)
	 *             fails
	 */
	public static IFile getFileFromProjectFolder(IProject project,
			String folderName, String fileName) throws CoreException {
		IFolder folder = project.getFolder(folderName);
		if (!folder.exists()) {
			folder.create(true, true, new NullProgressMonitor());
		}
		IFile file = ResourcesPlugin.getWorkspace().getRoot()
				.getFileForLocation(
						new Path(folder.getLocation() + "/" + fileName));//$NON-NLS-1$
		return file;
	}

	/**
	 * Writes the contents of a Writer to a specified file in a specified folder
	 * in a specified project. If the file exits, it overwrites the contents
	 * only if the flag, overwrite, is true.
	 * 
	 * @param writer
	 *            Source of the file content
	 * @param project
	 *            Handle to the project containing the folder,
	 *            <code>folderName</code> and the file <code>fileName</code>
	 * @param folderName
	 *            The name of the folder containing the file
	 *            <code>filename</code>
	 * @param fileName
	 *            The name of the file
	 * @param overwrite
	 *            Flag to indicate if an existing file should be over-written.
	 * @throws CoreException
	 */
	@SuppressWarnings("deprecation")
	public static void writeToProjectFile(Writer writer, IProject project,
			String folderName, String fileName, boolean overwrite)
			throws CoreException {
		if (writer.toString().equals("")) {
			System.out.println("*** No data in writer");
			return;
		}
		InputStream istream = new ByteArrayInputStream(writer.toString()
				.getBytes());

		ensureFolderExists(project, folderName);

		IFile file = getFileFromProjectFolder(project, folderName, fileName);
		if (!file.exists()) {
			file.create(istream, true, new NullProgressMonitor());
		} else if (overwrite) {
			file.setContents(istream, true, true, new NullProgressMonitor());
		} else {
			JMerger merger = getJMerger();
			merger.setSourceCompilationUnit(merger
					.createCompilationUnitForContents(writer.toString()));
			try {
				merger
						.setTargetCompilationUnit(merger
								.createCompilationUnitForInputStream(new FileInputStream(
										file.getLocation().toFile())));
			} catch (FileNotFoundException e) {
				System.out.println("*** " + fileName + " not found");
			}
			merger.merge();

			InputStream mergedContents = new ByteArrayInputStream(merger
					.getTargetCompilationUnitContents().getBytes());

			file.setContents(mergedContents, true, false,
					new NullProgressMonitor());
		}
	}

	/**
	 * Create the specified folder in the file system if it does not exist yet.
	 * If it does exist, then no action is taken.
	 * 
	 * @param project
	 *            is the handle to the project containing the folder,
	 *            <code>folderName</code>.
	 * @param folderName
	 *            is the name of the folder.
	 * @return A handle to the folder. If the folder does not exist, it will be
	 *         created
	 * @throws CoreException
	 *             If the creation of the folder (in the case it does not exist)
	 *             fails
	 */
	private static void ensureFolderExists(IProject project, String folderName)
			throws CoreException {
		IFolder folder = project.getFolder(folderName);
		if (!folder.exists()) {
			// make sure each subfolder exists as well as the full folder name
			// ensure that the folderName uses the File.separator
			folderName = folderName.replace('/', File.separator.charAt(0));
			folderName = folderName.replace('\\', File.separator.charAt(0));
			StringTokenizer strTok = new StringTokenizer(folderName,
					File.separator);
			String subFolderName = "";
			while (strTok.hasMoreTokens()) {
				subFolderName = subFolderName + strTok.nextToken();
				IFolder subFolder = project.getFolder(subFolderName);
				if (!subFolder.exists()) {
					subFolder.create(true, true, new NullProgressMonitor());
				}
				subFolderName += java.io.File.separator;
			}
			folder = project.getFolder(folderName);
			if (!folder.exists()) {
				// user must not have used the File.separator as subdir
				// separator.
				// Ensure that the folderName uses the File.separator
				folderName = folderName.replace('/', File.separator.charAt(0));
				folderName = folderName.replace('\\', File.separator.charAt(0));
				strTok = new StringTokenizer(folderName, File.separator);
				subFolderName = "";
				while (strTok.hasMoreTokens()) {
					subFolderName = subFolderName + strTok.nextToken();
					IFolder subFolder = project.getFolder(subFolderName);
					if (!subFolder.exists()) {
						subFolder.create(true, true, new NullProgressMonitor());
					}
					subFolderName += java.io.File.separator;
				}
				folder = project.getFolder(folderName);
			}
		}
	}

	/**
	 * Gets the project a model element is part of.
	 * 
	 * @param element
	 *            The model element. Checks assertion that <code>element</code>
	 *            is not null
	 * @return A handle to the project containing the model element
	 */
	public static IProject getProject(EObject element) {
		URI uri = element.eResource().getURI();
		IFile modelFile = ResourcesPlugin.getWorkspace().getRoot()
				.getFileForLocation(new Path(uri.device() + uri.path()));
		return modelFile.getProject();
	}

	public static String getPath(Package pkg) {
		String thePath = "";
		if (pkg != null) {
			Package nestingPackage = pkg.getNestingPackage();
			if ((nestingPackage != null)
					&& (!(nestingPackage instanceof Model))) {
				// nestingPackage.
				thePath = File.separator + nestingPackage.getName();
				// check to see if it has a parent package
				thePath += getPath(nestingPackage);
			}
		}
		return thePath;
	}

	public static void copyFile(String from, String to) throws IOException {
		File inputFile = new File(from);
		File outputFile = new File(to);

		FileInputStream in = new FileInputStream(inputFile);
		FileOutputStream out = new FileOutputStream(outputFile);
		int c;

		while ((c = in.read()) != -1)
			out.write(c);

		in.close();
		out.close();
	}

	public static void copyToProjectFile(String sourceFileName,
			IProject project, String folderName, String fileName,
			boolean overwrite) throws CoreException, FileNotFoundException {
		File inputFile = new File(sourceFileName);
		FileInputStream istream = new FileInputStream(inputFile);

		ensureFolderExists(project, folderName);

		IFile file = getFileFromProjectFolder(project, folderName, fileName);
		if (!file.exists()) {
			file.create(istream, true, new NullProgressMonitor());
		} else if (overwrite) {
			file.setContents(istream, true, true, new NullProgressMonitor());
		}
	}

	public static void generate(Object argument, JETEmitter emitter,
			IProject project, String folderName, String fileName,
			boolean overwrite) throws IOException, CoreException, JETException {
		Writer writer = new StringWriter();
		String output = emitter.generate(new NullProgressMonitor(),
				new Object[] { argument });
		writer.write(output);
		System.out.println("    Building " + folderName + File.separatorChar
				+ fileName);
		writeToProjectFile(writer, project, folderName, fileName, overwrite);
	}

	public static JETEmitter buildEmitter(String jetTemplate)
			throws Exception {
		String pluginId = "com.ibm.jps.uxtoweb";
		String base = Platform.getBundle(pluginId).getEntry("/").toString();
		String uri = base + jetTemplate;
		JETEmitter emitter = new JETEmitter(uri) {
		};
		emitter.addVariable("UML2", "org.eclipse.uml2.uml");
		emitter.addVariable("EMF_COMMON", "org.eclipse.emf.common");
		emitter.addVariable("EMF_ECORE", "org.eclipse.emf.ecore");
		return emitter;
	}

	/**
	 * Creates and initializes a new JMerger
	 * 
	 * @return JMerger
	 */
	@SuppressWarnings("deprecation")
	private static JMerger getJMerger() {
		JMerger jmerger = new JMerger();
		String uri = "";
		Bundle b = Platform.getBundle("com.ibm.jps.uxtoweb");
		if (b != null) {
			uri = b.getLocation();
			int pos = uri.indexOf(":/");
			if (pos != -1)
				uri = uri.substring(pos - 1);
		}
		uri += "templates/jmerge-rules.xml";
		JControlModel controlModel = new JControlModel(uri);
		jmerger.setControlModel(controlModel);
		return jmerger;
	}

	@SuppressWarnings("deprecation")
	public static void addLibrariesToProject(List libList,
			String srcFolderName, IProject project, String toFolderName)
			throws Exception {
//		System.out.println(">>> addLibrariesToProject: srcFolderName  = "
//				+ srcFolderName + ", toFolderName = " + toFolderName + ", project = " + project.getName());
		IJavaProject jp = JavaCore.create(project);
		if (!jp.exists())
			return;
		Bundle b = Platform.getBundle("com.ibm.jps.uxtoweb");
		IClasspathEntry[] existingEntries = jp.getRawClasspath();
		int pos = existingEntries.length;
		IClasspathEntry[] newEntries = new IClasspathEntry[pos + libList.size()];
		System.arraycopy(existingEntries, 0, newEntries, 0, pos);
		String path = File.separatorChar + project.getName()
				+ File.separatorChar + toFolderName;
		for (Iterator i = libList.iterator(); i.hasNext();) {
			String libName = (String) i.next();
//			System.out.println("    adding " + libName);
			Enumeration e = b.findEntries("/" + srcFolderName, libName, false);
			if (e != null && e.hasMoreElements()) {
				copyToProjectFile(Platform.resolve((URL) e.nextElement())
						.getPath().substring(1), project, toFolderName,
						libName, false);
				newEntries[pos++] = JavaCore.newLibraryEntry(new Path(path
						+ File.separatorChar + libName), null, // no source
						null, // no source
						false); // not exported
			}
		}
		jp.setRawClasspath(newEntries, null);
	}

	@SuppressWarnings("deprecation")
	public static void addFilesToProject(List fileList,
			String srcFolderName, IProject project, String toFolderName)
			throws Exception {
//		System.out.println(">>> addFilesToProject: srcFolderName  = "
//				+ srcFolderName + ", toFolderName = " + toFolderName + ", project = " + project.getName());
		Bundle b = Platform.getBundle("com.ibm.jps.uxtoweb");
		for (Iterator i = fileList.iterator(); i.hasNext();) {
			String fileName = (String) i.next();
//			System.out.println("    adding " + fileName);
			Enumeration e = b.findEntries("/" + srcFolderName, fileName, false);
			if (e != null && e.hasMoreElements()) {
				copyToProjectFile(Platform.resolve((URL) e.nextElement())
						.getPath().substring(1), project, toFolderName,
						fileName, false);
			}
		}
	}
	
	public static boolean isLibraryIncluded(String libName, IProject project)
			throws JavaModelException {
		IJavaProject jp = JavaCore.create(project);
		if (!jp.exists())
			return true;
		IPackageFragmentRoot[] pfr = jp.getAllPackageFragmentRoots();
		for (int i = 0; i < pfr.length; i++) {
			if (pfr[i].getElementName().equals(libName))
				return true;
		}
		return false;
	}
}